/*******************************************************************************
Program Name  : AJSPLIT.PRG
Purpose       : Split Big Files Into Smaller Chunks For Easy Storing
Written By    : Andi Jahja <andij@aonlippo.co.id>
Compiler      : Clipper Version 5.x
Compile       : Clipper AJSPLIT /n /w /a /m /l /es2
Link          : RTLINK/BLINKER/EXOSPACE file AJSPLIT
Syntax        : AJSPLIT <csource> [csplit] [nbyte]
Returns       : Logical .T. If Operation Suceed, .F. Otherwise.
Status        : Public Domain
Uploaded      : October 17, 1998
About AJSPLIT : AJSPLIT is a simple Clipper routine utilizing F*() functions to
                "break", or split big files into smaller chunks so that one can
                make a back up onto diskettes. This routine will create small
                files as instructed at the size of nbyte bytes each. For exam-
                ple, if one has a 12 MB files to be split into 300 K each,
                issue a command line such as : AJSPLIT MYFILE.DBF BACKUP 300000
                The files created will be as follows :
                1) Join.Bat
                2) Backup.1 ....... Backup.40
                To restore the files, put all the Backup.* and Join.bat onto
                a directory of whatever and type JOIN at C:> prompt.
                No time stamp synchronization is provided here since it will
                require third party library (Clipper did not have ones).
History       : - Oct 17, 1998 - Initial Release
                - Oct 17, 1998 - Added file size checking. If source size
                  is smaller than chunk size, process canceled.
                - Oct 19, 1998 - Added Set Epoch Command - Y2K
                               - Changed nByte To KiloBytes
                               - Added REM Info In Join.Bat
                               - Added DOS File Checker
********************************************************************************/
#include "fileio.ch"
********************************************************************************
function aj_split ( csource, csplit, nbyte )
********************************************************************************
local i                           // general counter
local ccommand      := ""         // dos command for joining files
local cexist        := ""         // batch file error checker
local nbufsize      := 8          // 8k for buffer Read/Write
local hsource       := 0          // file handle for source file
local hdestination  := 0          // file handle for destination file
local cbuffer       := ""         // buffer for read/write
local lsplit        := .f.        // return value
local nblock        := 0          // bytes read
local ncurrent      := 0          // total bytes copied
local nsplit        := 1          // destination file name extension
local cbat          := "join.bat" // for joining split files
local ctmp          := "@echo off" + chr(13) + chr(10) // 1st line in join.bat
local cdestination                // destination filename
local hbat                        // file handle for join.bat
local afile         := {}         // for information upon completion
local nseconds      := seconds()  // time elapsed
local nfilesize     := 0          // file size to be split
local hfile         := 0          // split file sizes

// y2k compliance
set(4,"dd/mm/yyyy")
set epoch to year(date()) - 50

// no params passed
if pcount() == 0
   ? "Usage : AJSPLIT <cSourceFile> [cSplitFileName] [nKBytesEach]"
   return lsplit
endif

// default destination name
csplit := if( csplit == nil, "split.", csplit + "." )

// default size of each split file 360 x 1024 bytes
nbyte := if( nbyte ==  nil, 368640, val( nbyte ) * 1024 )

// open the source file
if ( hsource := fopen(csource,FO_READ+FO_SHARED) ) <> -1
   // is file size smaller than chunk size ?
   if ( nfilesize := fseek( hsource, 0 , FS_END ) ) <= nbyte
      alert("***** Error *****;File Size Is Smaller Than Chunk Size;"+"Source Size = "+ltrim(str(nfilesize))+" Chunk Size = "+ltrim(str(nbyte)))
      fclose( hsource )
      return lsplit
   endif
   fseek( hsource, 0, FS_SET )               // go to top of file
   cdestination := csplit+ltrim(str(nsplit)) // destination file name
   hbat         := fcreate( cbat )           // join.bat
   ctmp         += "rem source file " + csource + " size "+ ltrim(str(nfilesize ) ) + chr(13) + chr(10)
   ctmp         += "rem split on " + dtoc(date()) + " "+ time() + chr(13) + chr(10)
   ccommand     := "copy /b "                // line in join.bat
   ccommand     += cdestination + "+"        // line in join.bat
   hdestination := fcreate(cdestination)     // create 1st split file
   nbufsize     *= 1024                      // buffer size
   cbuffer      := space( nbufsize )         // buffer read/write
   aadd( afile, cbat )
   aadd( afile, cdestination )
   dispoutat(24,00,padr("Writing " + cdestination,80))
   while !lsplit
      lsplit := ( ( ( nblock := fread( hsource, @cbuffer, nbufsize ) ) == 0 ) .or. ( fwrite ( hdestination, cbuffer, nblock ) < nblock ) )
      ncurrent += nblock
      if ncurrent >= nbyte                   // files size already exceed ?
         fclose( hdestination )              // close file
         ncurrent      := 0                  // reset counter
         cdestination  := csplit + ltrim(str(++nsplit))  // next file name
         ccommand      += cdestination + "+"             // line in join.bat
         hdestination  := fcreate(cdestination)          // create next file
         aadd( afile, cdestination )
         dispoutat(24,00,padr("Writing " + cdestination,80))
      endif
   enddo
   fclose( hsource )         // close source file
   fclose( hdestination )    // close split file
   ccommand := left( ccommand, rat("+",ccommand ) - 1 ) + " " // line in join.bat
   ccommand += csource + chr(13) + chr(10)                // line in join.bat
   ctmp += "rem the following files should be placed in a directory"+chr(13)+chr(10)
   for i := 2 to len( afile )
      hfile     := fopen( afile[i], FO_READ+FO_SHARED )
      nfilesize := fseek( hfile, 0 , FS_END )
      fclose( hfile )
      ctmp += "rem " + afile[i] + " - " + ltrim( str( nfilesize ) ) + chr(13) + chr(10)
   next
   for i := 2 to len( afile )                             // error checker
      ctmp += "if not exist " + afile[i] + " goto error"+ltrim(stR(i-1)) + chr(13) + chr(10)
   next
   ctmp += ccommand
   ctmp += "goto end" + chr(13) + chr(10)
   for i := 2 to len( afile )
      ctmp += ":error" + ltrim(str(i-1)) + chr(13) + chr(10)
      ctmp += "echo missing file " + afile[i] + chr(13) + chr(10)
      ctmp += "goto end" + chr(13) + chr(10)
   next
   ctmp += ":end" + chr(13) + chr(10)
   fwrite( hbat, upper( ctmp ) )   // write join.bat
   fclose( hbat )                  // close handle
   ? "Split succesful ..."
   ?
   ? "Files Created : "
   ?
   aeval( afile, { |e| qout(upper(e)) } )
   ?
   ? "Done in " + ltrim(str(seconds()-nseconds)) +" seconds."
   ? "To restore, type JOIN"
endif

return lsplit

*** EOF SPLIT.PRG ***
